use futures_util::{FutureExt, StreamExt};
use warp::Filter;

#[tokio::main]
async fn main() {
    // GET /hello/warp => 200 OK with body "Hello, warp!"
    let hello = warp::path!("hello" / String)
        .and(warp::header::<String>("user-agent"))
        .map(|name, agent| format!("Hello, {} from {}!", name, agent));

    // websocat ws://127.0.0.1:3030/ws-echo
    let echo = warp::path("ws-echo")
        .and(warp::ws())
        .map(|ws: warp::ws::Ws| {
            ws.on_upgrade(|websocket| {
                // echo all messages back
                let (tx, rx) = websocket.split();
                rx.forward(tx).map(|result| {
                    if let Err(e) = result {
                        eprintln!("websocket error: {:?}", e);
                    }
                })
            })
        });

    warp::serve(hello.or(echo))
        .run(([127, 0, 0, 1], 3030))
        .await;
}
